# Generated by Django 4.2.10 on 2024-03-06 09:29

import django.core.validators
import wbcore.models.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcompliance", "0013_alter_riskrule_unique_together"),
    ]

    operations = [
        migrations.AlterField(
            model_name="reviewcompliancetask",
            name="year",
            field=wbcore.models.fields.YearField(
                blank=True,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1000),
                    django.core.validators.MaxValueValidator(9999),
                ],
                verbose_name="Year",
            ),
        ),
    ]
