# Generated by Django 4.2.9 on 2024-02-26 13:47

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcompliance", "0012_alter_compliancetype_options"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="riskrule",
            unique_together=set(),  # we cannot have a uniqueness on these two fields because when the rule is created, it set the parameters with the default value until it is changed in a second step by the user. Furthermore, we kind of want user to be able to create similar rule but with different threashold and notifiee policy
        ),
    ]
