# Generated by Django 5.2.9 on 2025-12-16 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('contenttypes', '0002_remove_content_type_name'),
        ('directory', '0014_alter_entry_relationship_managers_and_more'),
        ('wbcompliance', '0022_riskincident_precheck'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='checkedobjectincidentrelationship',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='rulecheckedobjectrelationship',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='rulethreshold',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='checkedobjectincidentrelationship',
            constraint=models.UniqueConstraint(fields=('incident', 'rule_check'), name='unique_incident'),
        ),
        migrations.AddConstraint(
            model_name='rulecheckedobjectrelationship',
            constraint=models.UniqueConstraint(fields=('rule', 'checked_object_content_type', 'checked_object_id'), name='unique_checkedobjectrulerelationship'),
        ),
        migrations.AddConstraint(
            model_name='rulethreshold',
            constraint=models.UniqueConstraint(fields=('rule', 'severity'), name='unique_rule'),
        ),
    ]
