from .compliance_form import (
    CFComplianceFormSectionEndpointConfig,
    CFComplianceFormSignatureEndpointConfig,
    ComplianceFormEndpointConfig,
    ComplianceFormRuleEndpointConfig,
    ComplianceFormSectionRuleEndpointConfig,
    ComplianceFormSignatureEndpointConfig,
    ComplianceFormSignatureSectionRuleEndpointConfig,
    ComplianceFormTypeEndpointConfig,
)
from .compliance_task import (
    ComplianceActionEndpointConfig,
    ComplianceEventEndpointConfig,
    ComplianceTaskComplianceTaskGroupEndpointConfig,
    ComplianceTaskEndpointConfig,
    ComplianceTaskGroupEndpointConfig,
    ComplianceTaskInstanceComplianceTaskEndpointConfig,
    ComplianceTaskInstanceEndpointConfig,
    ComplianceTaskInstanceReviewGroupEndpointConfig,
    ComplianceTaskInstanceReviewNoGroupEndpointConfig,
    ComplianceTaskMatrixEndpointConfig,
    ComplianceTaskReviewGroupEndpointConfig,
    ComplianceTaskReviewNoGroupEndpointConfig,
    ReviewComplianceTaskEndpointConfig,
)
from .compliance_type import ComplianceTypeEndpointConfig
from .risk_managment import *
