# Generated by Django 4.2.7 on 2023-11-06 13:30

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "wbcompliance",
            "0001_initial_squashed_squashed_0010_alter_checkedobjectincidentrelationship_resolved_by_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="riskrule",
            name="parameters",
            field=models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder),
        ),
    ]
